@@ -133,6 +133,9 @@ func TestEvalAssign(t *testing.T) {
 		{src: "j := true || _", err: "1:33: cannot use _ as value"},
 		{src: "j := true && _", err: "1:33: cannot use _ as value"},
 		{src: "j := interface{}(int(1)); j.(_)", err: "1:54: cannot use _ as value"},
+		{src: "ff := func() (a, b, c int) {return 1, 2, 3}; x, y, x := ff()", err: "1:73: x repeated on left side of :="},
+		{src: "xx := 1; xx, _ := 2, 3", err: "1:37: no new variables on left side of :="},
+		{src: "1 = 2", err: "1:28: cannot assign to 1 (untyped int constant)"},
 	})
 }
 
@@ -216,6 +219,8 @@ func TestEvalTypeSpec(t *testing.T) {
 	runTests(t, i, []testCase{
 		{src: `type _ struct{}`, err: "1:19: cannot use _ as value"},
 		{src: `a := struct{a, _ int}{32, 0}`, res: "{32 0}"},
+		{src: "type A int; type A = string", err: "1:31: A redeclared in this block"},
+		{src: "type B int; type B string", err: "1:31: B redeclared in this block"},
 	})
 }
 
@@ -540,8 +545,8 @@ func TestEvalSliceExpression(t *testing.T) {
 		{src: `a := (&[]int{0,1,2,3})[1:3]`, err: "1:33: cannot slice type *[]int"},
 		{src: `a := "hello"[1:3:4]`, err: "1:45: invalid operation: 3-index slice of string"},
 		{src: `ar := [3]int{0,1,2}; a := ar[:4]`, err: "1:58: index int is out of bounds"},
-		{src: `a := []int{0,1,2,3}[1::4]`, err: "1:49: 2nd index required in 3-index slice"},
-		{src: `a := []int{0,1,2,3}[1:3:]`, err: "1:51: 3rd index required in 3-index slice"},
+		{src: `a := []int{0,1,2,3}[1::4]`, err: "index required in 3-index slice"},
+		{src: `a := []int{0,1,2,3}[1:3:]`, err: "index required in 3-index slice"},
 		{src: `a := []int{0,1,2}[3:1]`, err: "invalid index values, must be low <= high <= max"},
 		{pre: func() { eval(t, i, `type Str = string; var r Str = "truc"`) }, src: `r[1]`, res: "114"},
 		{src: `_[12]`, err: "1:28: cannot use _ as value"},
@@ -640,6 +645,8 @@ func TestEvalChan(t *testing.T) {
 				return ok && msg == "ping"
 			})()`, res: "true",
 		},
+		{src: `a :=5; a <- 4`, err: "cannot send to non-channel int"},
+		{src: `a :=5; b := <-a`, err: "cannot receive from non-channel int"},
 	})
 }
 
