@@ -17,7 +17,6 @@ type tcat uint
 // Types for go language.
 const (
 	nilT tcat = iota
-	aliasT
 	arrayT
 	binT
 	binPkgT
@@ -41,6 +40,7 @@ const (
 	int16T
 	int32T
 	int64T
+	linkedT
 	mapT
 	ptrT
 	sliceT
@@ -60,7 +60,6 @@ const (
 
 var cats = [...]string{
 	nilT:        "nilT",
-	aliasT:      "aliasT",
 	arrayT:      "arrayT",
 	binT:        "binT",
 	binPkgT:     "binPkgT",
@@ -75,14 +74,15 @@ var cats = [...]string{
 	float32T:    "float32",
 	float64T:    "float64T",
 	funcT:       "funcT",
+	genericT:    "genericT",
 	interfaceT:  "interfaceT",
 	intT:        "intT",
 	int8T:       "int8T",
 	int16T:      "int16T",
 	int32T:      "int32T",
 	int64T:      "int64T",
+	linkedT:     "linkedT",
 	mapT:        "mapT",
-	genericT:    "genericT",
 	ptrT:        "ptrT",
 	sliceT:      "sliceT",
 	srcPkgT:     "srcPkgT",
@@ -118,7 +118,7 @@ type itype struct {
 	cat          tcat          // Type category
 	field        []structField // Array of struct fields if structT or interfaceT
 	key          *itype        // Type of key element if MapT or nil
-	val          *itype        // Type of value element if chanT, chanSendT, chanRecvT, mapT, ptrT, aliasT, arrayT, sliceT, variadicT or genericT
+	val          *itype        // Type of value element if chanT, chanSendT, chanRecvT, mapT, ptrT, linkedT, arrayT, sliceT, variadicT or genericT
 	recv         *itype        // Receiver type for funcT or nil
 	arg          []*itype      // Argument types if funcT or nil
 	ret          []*itype      // Return types if funcT or nil
@@ -126,6 +126,7 @@ type itype struct {
 	method       []*node       // Associated methods or nil
 	constraint   []*itype      // For interfaceT: list of types part of interface set
 	ulconstraint []*itype      // For interfaceT: list of underlying types part of interface set
+	instance     []*itype      // For genericT: list of instantiated types
 	name         string        // name of type within its package for a defined type
 	path         string        // for a defined type, the package import path
 	length       int           // length of array if ArrayT
@@ -248,7 +249,7 @@ func namedOf(val *itype, path, name string, opts ...itypeOption) *itype {
 	if path != "" {
 		str = path + "." + name
 	}
-	t := &itype{cat: aliasT, val: val, path: path, name: name, str: str}
+	t := &itype{cat: linkedT, val: val, path: path, name: name, str: str}
 	for _, opt := range opts {
 		opt(t)
 	}
@@ -373,8 +374,8 @@ func structOf(t *itype, fields []structField, opts ...itypeOption) *itype {
 }
 
 // genericOf returns a generic type.
-func genericOf(val *itype, name string, opts ...itypeOption) *itype {
-	t := &itype{cat: genericT, name: name, str: name, val: val}
+func genericOf(val *itype, name, path string, opts ...itypeOption) *itype {
+	t := &itype{cat: genericT, name: name, path: path, str: name, val: val}
 	for _, opt := range opts {
 		opt(t)
 	}
@@ -446,12 +447,19 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 		)
 		switch v := c0.rval; {
 		case v.IsValid():
-			// Size if defined by a constant litteral value.
+			// Size if defined by a constant literal value.
 			if isConstantValue(v.Type()) {
 				c := v.Interface().(constant.Value)
 				length = constToInt(c)
 			} else {
-				length = int(v.Int())
+				switch v.Type().Kind() {
+				case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
+					length = int(v.Int())
+				case reflect.Uint, reflect.Uint8, reflect.Uint16, reflect.Uint32, reflect.Uint64, reflect.Uintptr:
+					length = int(v.Uint())
+				default:
+					return nil, c0.cfgErrorf("non integer constant %v", v)
+				}
 			}
 		case c0.kind == ellipsisExpr:
 			// [...]T expression, get size from the length of composite array.
@@ -476,7 +484,11 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 			length = int(vInt(sym.rval))
 		default:
 			// Size is defined by a numeric constant expression.
-			if _, err = interp.cfg(c0, sc, sc.pkgID, sc.pkgName); err != nil {
+			if _, err := interp.cfg(c0, sc, sc.pkgID, sc.pkgName); err != nil {
+				if strings.Contains(err.Error(), " undefined: ") {
+					incomplete = true
+					break
+				}
 				return nil, err
 			}
 			v, ok := c0.rval.Interface().(constant.Value)
@@ -587,7 +599,10 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 		}
 
 		if isInterfaceSrc(dt) {
-			dt.val = t
+			// Set a new interface type preserving the concrete type (.val field).
+			t2 := *dt
+			t2.val = t
+			dt = &t2
 		}
 		t = dt
 
@@ -768,11 +783,16 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 			}
 		}
 		if sym.kind == varTypeSym {
-			t = genericOf(sym.typ, n.ident, withNode(n), withScope(sc))
+			t = genericOf(sym.typ, n.ident, sc.pkgName, withNode(n), withScope(sc))
 		} else {
 			t = sym.typ
 		}
-		if t.incomplete && t.cat == aliasT && t.val != nil && t.val.cat != nilT {
+		if t == nil {
+			if t, err = nodeType2(interp, sc, sym.node, seen); err != nil {
+				return nil, err
+			}
+		}
+		if t.incomplete && t.cat == linkedT && t.val != nil && t.val.cat != nilT {
 			t.incomplete = false
 		}
 		if t.incomplete && t.node != n {
@@ -793,7 +813,11 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 			return nil, err
 		}
 		if lt.incomplete {
-			t.incomplete = true
+			if t == nil {
+				t = lt
+			} else {
+				t.incomplete = true
+			}
 			break
 		}
 		switch lt.cat {
@@ -814,46 +838,59 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 				break
 			}
 			// A generic type is being instantiated. Generate it.
-			g, err := genAST(sc, lt.node.anc, []*node{t1.node})
+			t, err = genType(interp, sc, name, lt, []*itype{t1}, seen)
 			if err != nil {
 				return nil, err
 			}
-			t, err = nodeType2(interp, sc, g.lastChild(), seen)
-			if err != nil {
-				return nil, err
-			}
-			sc.sym[name] = &symbol{index: -1, kind: typeSym, typ: t, node: g}
-
-			// Instantiate type methods (if any).
-			var pt *itype
-			if len(lt.method) > 0 {
-				pt = ptrOf(t, withNode(g), withScope(sc))
-			}
-			for _, nod := range lt.method {
-				gm, err := genAST(sc, nod, []*node{t1.node})
-				if err != nil {
-					return nil, err
-				}
-				if gm.typ, err = nodeType(interp, sc, gm.child[2]); err != nil {
-					return nil, err
-				}
-				t.addMethod(gm)
-				if rtn := gm.child[0].child[0].lastChild(); rtn.kind == starExpr {
-					// The receiver is a pointer on a generic type.
-					pt.addMethod(gm)
-					rtn.typ = pt
-				}
-				// Compile method CFG.
-				if _, err = interp.cfg(gm, sc, sc.pkgID, sc.pkgName); err != nil {
-					return nil, err
-				}
-				// Generate closures for function body.
-				if err = genRun(gm); err != nil {
-					return nil, err
-				}
-			}
 		}
 
+	case indexListExpr:
+		// Similar to above indexExpr for generic types, but handle multiple type parameters.
+		var lt *itype
+		if lt, err = nodeType2(interp, sc, n.child[0], seen); err != nil {
+			return nil, err
+		}
+		if lt.incomplete {
+			if t == nil {
+				t = lt
+			} else {
+				t.incomplete = true
+			}
+			break
+		}
+
+		// Index list expressions can be used only in context of generic types.
+		if lt.cat != genericT {
+			err = n.cfgErrorf("not a generic type: %s", lt.id())
+			return nil, err
+		}
+		name := lt.id() + "["
+		out := false
+		types := []*itype{}
+		for _, c := range n.child[1:] {
+			t1, err := nodeType2(interp, sc, c, seen)
+			if err != nil {
+				return nil, err
+			}
+			if t1.cat == genericT || t1.incomplete {
+				t = lt
+				out = true
+				break
+			}
+			types = append(types, t1)
+			name += t1.id() + ","
+		}
+		if out {
+			break
+		}
+		name = strings.TrimSuffix(name, ",") + "]"
+		if sym, _, found := sc.lookup(name); found {
+			t = sym.typ
+			break
+		}
+		// A generic type is being instantiated. Generate it.
+		t, err = genType(interp, sc, name, lt, types, seen)
+
 	case interfaceType:
 		if sname := typeName(n); sname != "" {
 			if sym, _, found := sc.lookup(sname); found && sym.kind == typeSym {
@@ -998,7 +1035,7 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 		sname := structName(n)
 		if sname != "" {
 			sym, _, found = sc.lookup(sname)
-			if found && sym.kind == typeSym {
+			if found && sym.kind == typeSym && sym.typ != nil {
 				t = structOf(sym.typ, sym.typ.field, withNode(n), withScope(sc))
 			} else {
 				t = structOf(nil, nil, withNode(n), withScope(sc))
@@ -1044,6 +1081,9 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 		t = structOf(t, fields, withNode(n), withScope(sc))
 		t.incomplete = incomplete
 		if sname != "" {
+			if sc.sym[sname] == nil {
+				sc.sym[sname] = &symbol{index: -1, kind: typeSym, node: n}
+			}
 			sc.sym[sname].typ = t
 		}
 
@@ -1076,6 +1116,58 @@ func nodeType2(interp *Interpreter, sc *scope, n *node, seen []*node) (t *itype,
 	return t, err
 }
 
+func genType(interp *Interpreter, sc *scope, name string, lt *itype, types []*itype, seen []*node) (t *itype, err error) {
+	// A generic type is being instantiated. Generate it.
+	g, _, err := genAST(sc, lt.node.anc, types)
+	if err != nil {
+		return nil, err
+	}
+	t, err = nodeType2(interp, sc, g.lastChild(), seen)
+	if err != nil {
+		return nil, err
+	}
+	lt.instance = append(lt.instance, t)
+	// Add generated symbol in the scope of generic source and user.
+	sc.sym[name] = &symbol{index: -1, kind: typeSym, typ: t, node: g}
+	if lt.scope.sym[name] == nil {
+		lt.scope.sym[name] = sc.sym[name]
+	}
+
+	for _, nod := range lt.method {
+		if err := genMethod(interp, sc, t, nod, types); err != nil {
+			return nil, err
+		}
+	}
+	return t, err
+}
+
+func genMethod(interp *Interpreter, sc *scope, t *itype, nod *node, types []*itype) error {
+	gm, _, err := genAST(sc, nod, types)
+	if err != nil {
+		return err
+	}
+	if gm.typ, err = nodeType(interp, sc, gm.child[2]); err != nil {
+		return err
+	}
+	t.addMethod(gm)
+
+	// If the receiver is a pointer to a generic type, generate also the pointer type.
+	if rtn := gm.child[0].child[0].lastChild(); rtn != nil && rtn.kind == starExpr {
+		pt := ptrOf(t, withNode(t.node), withScope(sc))
+		pt.addMethod(gm)
+		rtn.typ = pt
+	}
+
+	// Compile the method AST in the scope of the generic type.
+	scop := nod.typ.scope
+	if _, err = interp.cfg(gm, scop, scop.pkgID, scop.pkgName); err != nil {
+		return err
+	}
+
+	// Generate closures for function body.
+	return genRun(gm)
+}
+
 // findPackageType searches the top level scope for a package type.
 func findPackageType(interp *Interpreter, sc *scope, n *node) *itype {
 	// Find the root scope, the package symbols will exist there.
@@ -1242,8 +1334,13 @@ func (t *itype) numOut() int {
 		if t.rtype.Kind() == reflect.Func {
 			return t.rtype.NumOut()
 		}
+	case builtinT:
+		switch t.name {
+		case "append", "cap", "complex", "copy", "imag", "len", "make", "new", "real", "recover":
+			return 1
+		}
 	}
-	return 1
+	return 0
 }
 
 func (t *itype) out(i int) *itype {
@@ -1266,7 +1363,7 @@ func (t *itype) concrete() *itype {
 }
 
 func (t *itype) underlying() *itype {
-	if t.cat == aliasT {
+	if t.cat == linkedT {
 		return t.val.underlying()
 	}
 	return t
@@ -1274,10 +1371,10 @@ func (t *itype) underlying() *itype {
 
 // typeDefined returns true if type t1 is defined from type t2 or t2 from t1.
 func typeDefined(t1, t2 *itype) bool {
-	if t1.cat == aliasT && t1.val == t2 {
+	if t1.cat == linkedT && t1.val == t2 {
 		return true
 	}
-	if t2.cat == aliasT && t2.val == t1 {
+	if t2.cat == linkedT && t2.val == t1 {
 		return true
 	}
 	return false
@@ -1313,7 +1410,7 @@ func isComplete(t *itype, visited map[string]bool) bool {
 		visited[name] = true
 	}
 	switch t.cat {
-	case aliasT:
+	case linkedT:
 		if t.val != nil && t.val.cat != nilT {
 			// A type aliased to a partially defined type is considered complete, to allow recursivity.
 			return true
@@ -1357,7 +1454,7 @@ func (t *itype) assignableTo(o *itype) bool {
 		return true
 	}
 
-	if t.cat == aliasT && o.cat == aliasT && (t.underlying().id() != o.underlying().id() || !typeDefined(t, o)) {
+	if t.cat == linkedT && o.cat == linkedT && (t.underlying().id() != o.underlying().id() || !typeDefined(t, o)) {
 		return false
 	}
 
@@ -1472,7 +1569,7 @@ func (t *itype) methods() methodSet {
 		seen[typ] = true
 
 		switch typ.cat {
-		case aliasT:
+		case linkedT:
 			for k, v := range getMethods(typ.val) {
 				res[k] = v
 			}
@@ -1563,7 +1660,7 @@ func (t *itype) zero() (v reflect.Value, err error) {
 		return v, err
 	}
 	switch t.cat {
-	case aliasT:
+	case linkedT:
 		v, err = t.val.zero()
 
 	case arrayT, ptrT, structT, sliceT:
@@ -1581,7 +1678,7 @@ func (t *itype) zero() (v reflect.Value, err error) {
 // fieldIndex returns the field index from name in a struct, or -1 if not found.
 func (t *itype) fieldIndex(name string) int {
 	switch t.cat {
-	case aliasT, ptrT:
+	case linkedT, ptrT:
 		return t.val.fieldIndex(name)
 	}
 	for i, field := range t.field {
@@ -1608,6 +1705,7 @@ func (t *itype) fieldSeq(seq []int) *itype {
 func (t *itype) lookupField(name string) []int {
 	seen := map[*itype]bool{}
 	var lookup func(*itype) []int
+	tias := isStruct(t)
 
 	lookup = func(typ *itype) []int {
 		if seen[typ] {
@@ -1616,7 +1714,7 @@ func (t *itype) lookupField(name string) []int {
 		seen[typ] = true
 
 		switch typ.cat {
-		case aliasT, ptrT:
+		case linkedT, ptrT:
 			return lookup(typ.val)
 		}
 		if fi := typ.fieldIndex(name); fi >= 0 {
@@ -1625,7 +1723,12 @@ func (t *itype) lookupField(name string) []int {
 
 		for i, f := range typ.field {
 			switch f.typ.cat {
-			case ptrT, structT, interfaceT, aliasT:
+			case ptrT, structT, interfaceT, linkedT:
+				if tias != isStruct(f.typ) {
+					// Interface fields are not valid embedded struct fields.
+					// Struct fields are not valid interface fields.
+					break
+				}
 				if index2 := lookup(f.typ); len(index2) > 0 {
 					return append([]int{i}, index2...)
 				}
@@ -1684,7 +1787,7 @@ func (t *itype) methodCallType() reflect.Type {
 }
 
 func (t *itype) resolveAlias() *itype {
-	for t.cat == aliasT {
+	for t.cat == linkedT {
 		t = t.val
 	}
 	return t
@@ -1728,13 +1831,46 @@ func (t *itype) lookupMethod2(name string, seen map[*itype]bool) (*node, []int)
 				}
 			}
 		}
-		if t.cat == aliasT || isInterfaceSrc(t) && t.val != nil {
+		if t.cat == linkedT || isInterfaceSrc(t) && t.val != nil {
 			return t.val.lookupMethod2(name, seen)
 		}
 	}
 	return m, index
 }
 
+// interfaceMethod returns type of method matching an interface method name (not as a concrete method).
+func (t *itype) interfaceMethod(name string) *itype {
+	return t.interfaceMethod2(name, nil)
+}
+
+func (t *itype) interfaceMethod2(name string, seen map[*itype]bool) *itype {
+	if seen == nil {
+		seen = map[*itype]bool{}
+	}
+	if seen[t] {
+		return nil
+	}
+	seen[t] = true
+	if t.cat == ptrT {
+		return t.val.interfaceMethod2(name, seen)
+	}
+	for _, f := range t.field {
+		if f.name == name && isInterface(t) {
+			return f.typ
+		}
+		if !f.embed {
+			continue
+		}
+		if typ := f.typ.interfaceMethod2(name, seen); typ != nil {
+			return typ
+		}
+	}
+	if t.cat == linkedT || isInterfaceSrc(t) && t.val != nil {
+		return t.val.interfaceMethod2(name, seen)
+	}
+	return nil
+}
+
 // methodDepth returns a depth greater or equal to 0, or -1 if no match.
 func (t *itype) methodDepth(name string) int {
 	if m, lint := t.lookupMethod(name); m != nil {
@@ -1817,15 +1953,11 @@ func exportName(s string) string {
 
 var (
 	// TODO(mpl): generators.
-	interf   = reflect.TypeOf((*interface{})(nil)).Elem()
-	constVal = reflect.TypeOf((*constant.Value)(nil)).Elem()
+	emptyInterfaceType = reflect.TypeOf((*interface{})(nil)).Elem()
+	valueInterfaceType = reflect.TypeOf((*valueInterface)(nil)).Elem()
+	constVal           = reflect.TypeOf((*constant.Value)(nil)).Elem()
 )
 
-type fieldRebuild struct {
-	typ *itype
-	idx int
-}
-
 type refTypeContext struct {
 	defined map[string]*itype
 
@@ -1833,7 +1965,7 @@ type refTypeContext struct {
 	// type name (as key) is used as a field of another (or possibly the same) struct
 	// type. Each of these fields will then live as an unsafe2.dummy type until the
 	// whole recursion is fully resolved, and the type is fixed.
-	refs map[string][]fieldRebuild
+	refs map[string][]*itype
 
 	// When we detect for the first time that we are in a recursive type (thanks to
 	// defined), we keep track of the first occurrence of the type where the recursion
@@ -1841,6 +1973,7 @@ type refTypeContext struct {
 	// "top-level" point.
 	rect       *itype
 	rebuilding bool
+	slevel     int
 }
 
 // Clone creates a copy of the ref type context.
@@ -1905,7 +2038,7 @@ func (t *itype) refType(ctx *refTypeContext) reflect.Type {
 	if ctx == nil {
 		ctx = &refTypeContext{
 			defined: map[string]*itype{},
-			refs:    map[string][]fieldRebuild{},
+			refs:    map[string][]*itype{},
 		}
 	}
 	if t.incomplete || t.cat == nilT {
@@ -1930,21 +2063,21 @@ func (t *itype) refType(ctx *refTypeContext) reflect.Type {
 
 		// The recursion has not been fully resolved yet.
 		// To indicate that a rebuild is needed on the englobing struct,
-		// return a dummy field type and create an entry with an empty fieldRebuild.
+		// return a dummy field type and create an empty entry.
 		flds := ctx.refs[name]
 		ctx.rect = dt
 
 		// We know we are used as a field by someone, but we don't know by who
-		// at this point in the code, so we just mark it as an empty fieldRebuild for now.
-		// We'll complete the fieldRebuild in the caller.
-		ctx.refs[name] = append(flds, fieldRebuild{})
+		// at this point in the code, so we just mark it as an empty *itype for now.
+		// We'll complete the *itype in the caller.
+		ctx.refs[name] = append(flds, (*itype)(nil))
 		return unsafe2.DummyType
 	}
 	if isGeneric(t) {
 		return reflect.TypeOf((*generic)(nil)).Elem()
 	}
 	switch t.cat {
-	case aliasT:
+	case linkedT:
 		t.rtype = t.val.refType(ctx)
 	case arrayT:
 		t.rtype = reflect.ArrayOf(t.length, t.val.refType(ctx))
@@ -1971,17 +2104,30 @@ func (t *itype) refType(ctx *refTypeContext) reflect.Type {
 		}
 		t.rtype = reflect.FuncOf(in, out, variadic)
 	case interfaceT:
-		t.rtype = interf
+		if len(t.field) == 0 {
+			// empty interface, do not wrap it
+			t.rtype = emptyInterfaceType
+			break
+		}
+		t.rtype = valueInterfaceType
 	case mapT:
 		t.rtype = reflect.MapOf(t.key.refType(ctx), t.val.refType(ctx))
 	case ptrT:
-		t.rtype = reflect.PtrTo(t.val.refType(ctx))
+		rt := t.val.refType(ctx)
+		if rt == unsafe2.DummyType && ctx.slevel > 1 {
+			// We have a pointer to a recursive struct which is not yet fully computed.
+			// Return it but do not yet store it in rtype, so the complete version can
+			// be stored in future.
+			return reflect.PtrTo(rt)
+		}
+		t.rtype = reflect.PtrTo(rt)
 	case structT:
 		if t.name != "" {
 			ctx.defined[name] = t
 		}
+		ctx.slevel++
 		var fields []reflect.StructField
-		for i, f := range t.field {
+		for _, f := range t.field {
 			field := reflect.StructField{
 				Name: exportName(f.name), Type: f.typ.refType(ctx),
 				Tag: reflect.StructTag(f.tag), Anonymous: f.embed,
@@ -1990,13 +2136,18 @@ func (t *itype) refType(ctx *refTypeContext) reflect.Type {
 			// Find any nil type refs that indicates a rebuild is needed on this field.
 			for _, flds := range ctx.refs {
 				for j, fld := range flds {
-					if fld.typ == nil {
-						flds[j] = fieldRebuild{typ: t, idx: i}
+					if fld == nil {
+						flds[j] = t
 					}
 				}
 			}
 		}
-		fieldFix := []int{} // Slice of field indices to fix for recursivity.
+		ctx.slevel--
+		type fixStructField struct {
+			name  string
+			index int
+		}
+		fieldFix := []fixStructField{} // Slice of field indices to fix for recursivity.
 		t.rtype = reflect.StructOf(fields)
 		if ctx.isComplete() {
 			for _, s := range ctx.defined {
@@ -2005,8 +2156,12 @@ func (t *itype) refType(ctx *refTypeContext) reflect.Type {
 					if strings.HasSuffix(f.Type.String(), "unsafe2.dummy") {
 						unsafe2.SetFieldType(s.rtype, i, ctx.rect.fixDummy(s.rtype.Field(i).Type))
 						if name == s.path+"/"+s.name {
-							fieldFix = append(fieldFix, i)
+							fieldFix = append(fieldFix, fixStructField{s.name, i})
 						}
+						continue
+					}
+					if f.Type.Kind() == reflect.Func && strings.Contains(f.Type.String(), "unsafe2.dummy") {
+						fieldFix = append(fieldFix, fixStructField{s.name, i})
 					}
 				}
 			}
@@ -2021,9 +2176,11 @@ func (t *itype) refType(ctx *refTypeContext) reflect.Type {
 		// and we need both the loop above, around all the struct fields, and the loop
 		// below, around the ctx.refs.
 		for _, f := range ctx.refs[name] {
-			for _, index := range fieldFix {
-				ftyp := f.typ.field[index].typ.refType(&refTypeContext{defined: ctx.defined, rebuilding: true})
-				unsafe2.SetFieldType(f.typ.rtype, index, ftyp)
+			for _, ff := range fieldFix {
+				if ff.name == f.name {
+					ftyp := f.field[ff.index].typ.refType(&refTypeContext{defined: ctx.defined, rebuilding: true})
+					unsafe2.SetFieldType(f.rtype, ff.index, ftyp)
+				}
 			}
 		}
 	default:
@@ -2045,21 +2202,19 @@ func (t *itype) frameType() (r reflect.Type) {
 		panic(err)
 	}
 	switch t.cat {
-	case aliasT:
+	case linkedT:
 		r = t.val.frameType()
 	case arrayT:
 		r = reflect.ArrayOf(t.length, t.val.frameType())
 	case sliceT, variadicT:
 		r = reflect.SliceOf(t.val.frameType())
-	case funcT:
-		r = reflect.TypeOf((*node)(nil))
 	case interfaceT:
 		if len(t.field) == 0 {
 			// empty interface, do not wrap it
-			r = reflect.TypeOf((*interface{})(nil)).Elem()
+			r = emptyInterfaceType
 			break
 		}
-		r = reflect.TypeOf((*valueInterface)(nil)).Elem()
+		r = valueInterfaceType
 	case mapT:
 		r = reflect.MapOf(t.key.frameType(), t.val.frameType())
 	case ptrT:
@@ -2072,6 +2227,14 @@ func (t *itype) frameType() (r reflect.Type) {
 
 func (t *itype) implements(it *itype) bool {
 	if isBin(t) {
+		// Note: in case of a valueInterfaceType, we
+		// miss required data which will be available
+		// later, so we optimistically return true to progress,
+		// and additional checks will be hopefully performed at
+		// runtime.
+		if rt := it.TypeOf(); rt == valueInterfaceType {
+			return true
+		}
 		return t.TypeOf().Implements(it.TypeOf())
 	}
 	return t.methods().contains(it.methods())
@@ -2127,11 +2290,15 @@ func (t *itype) defaultType(v reflect.Value, sc *scope) *itype {
 func (t *itype) isNil() bool { return t.cat == nilT }
 
 func (t *itype) hasNil() bool {
-	switch t.TypeOf().Kind() {
+	switch rt := t.TypeOf(); rt.Kind() {
 	case reflect.UnsafePointer:
 		return true
 	case reflect.Slice, reflect.Ptr, reflect.Func, reflect.Interface, reflect.Map, reflect.Chan:
 		return true
+	case reflect.Struct:
+		if rt == valueInterfaceType {
+			return true
+		}
 	}
 	return false
 }
@@ -2164,16 +2331,6 @@ func constToString(v reflect.Value) string {
 	return constant.StringVal(c)
 }
 
-func defRecvType(n *node) *itype {
-	if n.kind != funcDecl || len(n.child[0].child) == 0 {
-		return nil
-	}
-	if r := n.child[0].child[0].lastChild(); r != nil {
-		return r.typ
-	}
-	return nil
-}
-
 func wrappedType(n *node) *itype {
 	if n.typ.cat != valueT {
 		return nil
@@ -2192,7 +2349,7 @@ func isShiftNode(n *node) bool {
 // chanElement returns the channel element type.
 func chanElement(t *itype) *itype {
 	switch t.cat {
-	case aliasT:
+	case linkedT:
 		return chanElement(t.val)
 	case chanT, chanSendT, chanRecvT:
 		return t.val
@@ -2216,12 +2373,16 @@ func isGeneric(t *itype) bool {
 	return t.cat == funcT && t.node != nil && len(t.node.child) > 0 && len(t.node.child[0].child) > 0
 }
 
+func isNamedFuncSrc(t *itype) bool {
+	return isFuncSrc(t) && t.node.anc.kind == funcDecl
+}
+
 func isFuncSrc(t *itype) bool {
-	return t.cat == funcT || (t.cat == aliasT && isFuncSrc(t.val))
+	return t.cat == funcT || (t.cat == linkedT && isFuncSrc(t.val))
 }
 
 func isPtrSrc(t *itype) bool {
-	return t.cat == ptrT || (t.cat == aliasT && isPtrSrc(t.val))
+	return t.cat == ptrT || (t.cat == linkedT && isPtrSrc(t.val))
 }
 
 func isSendChan(t *itype) bool {
@@ -2238,7 +2399,7 @@ func isArray(t *itype) bool {
 }
 
 func isInterfaceSrc(t *itype) bool {
-	return t.cat == interfaceT || (t.cat == aliasT && isInterfaceSrc(t.val))
+	return t.cat == interfaceT || (t.cat == linkedT && isInterfaceSrc(t.val))
 }
 
 func isInterfaceBin(t *itype) bool {
@@ -2246,14 +2407,14 @@ func isInterfaceBin(t *itype) bool {
 }
 
 func isInterface(t *itype) bool {
-	return isInterfaceSrc(t) || t.TypeOf() != nil && t.TypeOf().Kind() == reflect.Interface
+	return isInterfaceSrc(t) || t.TypeOf() == valueInterfaceType || t.TypeOf() != nil && t.TypeOf().Kind() == reflect.Interface
 }
 
 func isBin(t *itype) bool {
 	switch t.cat {
 	case valueT:
 		return true
-	case aliasT, ptrT:
+	case linkedT, ptrT:
 		return isBin(t.val)
 	default:
 		return false
@@ -2266,7 +2427,7 @@ func isStruct(t *itype) bool {
 	switch t.cat {
 	case structT:
 		return true
-	case aliasT, ptrT:
+	case linkedT, ptrT:
 		return isStruct(t.val)
 	case valueT:
 		k := t.rtype.Kind()
